# -*- coding: utf-8 -*-

# Standard Library
import os
import json

# Vendor Library
from simple_report.report import SpreadsheetReport
from m3.actions.exceptions import ApplicationLogicException

# Project Library
from kinder.core.dict.models import DouType
from gisdo.index.constants import (
    APPLICATION_INDEXES, QUEUE_INDEXES,
    ENROLLED_INDEXES, CAPACITIES_INDEXES)
from gisdo import settings as gs_settings
from gisdo import constants as const
from gisdo.models import GisdoUnit


class ReportFormReport(object):

    APPLICATION = 'applications'
    QUEUE = 'queues'
    ENROLLED = 'enrolled'
    CAPACITIES = 'capacities'

    PARTS = {
        APPLICATION: APPLICATION_INDEXES,
        QUEUE: QUEUE_INDEXES,
        ENROLLED: ENROLLED_INDEXES,
        CAPACITIES: CAPACITIES_INDEXES
    }

    def __init__(self, result_name):

        templates = os.path.abspath(
            os.path.join(os.path.dirname(__file__), "templates"))
        self.template_name = os.path.join(
            templates, "report_template.xlsx")
        self.result_name = os.path.join(
            gs_settings.DOWNLOADS_DIR, result_name + ".xlsx")

        self.report = SpreadsheetReport(self.template_name)

    def flush_header(self, full_name, email, phone=''):

        header = self.report.get_section('header')
        header.flush({
            'full_name': full_name,
            'email': email,
            'phone': phone
        })

    def collect_row_data(self, report_form_row, date, ocato):

        mo = report_form_row.unit.get_mo()
        try:
            doo_id = report_form_row.unit.gisdo.doo_identity
        except GisdoUnit.DoesNotExist:
            raise ApplicationLogicException(
                u'Учреждение c ID=%d не найдено в '
                u'модуле "Фед. отчетности"' %
                report_form_row.unit.pk)
        unit_dou_type = report_form_row.unit.dou_type
        if not unit_dou_type:
            unit_dou_type = DouType.objects.get(code=DouType.GOVERNMENT)

        row_data = {
            'mo': mo.name,
            'dou': report_form_row.unit.name,
            'date': str(date),
            'okato': ocato,
            'octmo': mo.octmo,
            'doo_id': doo_id,
            'dou_type': unit_dou_type.name
        }

        row_data.update(ReportFormReport._collect_data(
            report_form_row, index_group=ReportFormReport.APPLICATION))
        row_data.update(ReportFormReport._collect_data(
            report_form_row, index_group=ReportFormReport.QUEUE))
        row_data.update(ReportFormReport._collect_data(
            report_form_row, index_group=ReportFormReport.ENROLLED))
        row_data.update(ReportFormReport._collect_data(
            report_form_row, index_group=ReportFormReport.CAPACITIES))

        return row_data

    def flush_row_data(self, data):

        row = self.report.get_section('row')

        for portion in data:
            row.flush(portion)

    def build(self):

        self.report.build(self.result_name)

    @staticmethod
    def _collect_data(report_form_row, index_group):

        data = json.loads(
            getattr(
                report_form_row, index_group))

        result = {}

        age_slices = (
            const.AGE_CATEGORIES_FULL,
            const.AGE_CATEGORIES_EIGHT,
            const.AGE_CATEGORIES_CUT,
            {'ALL': const.ALL})

        for index_id in ReportFormReport.PARTS[index_group]:
            for age_slice in age_slices:
                for age_category in age_slice:
                    index = data[index_id]
                    if age_slice[age_category] in index:
                        report_index_id = index_id.replace('.', '_')
                        report_age_cat = age_category.replace(
                            '-', '_').replace(
                                '.', '_')
                        result[
                            report_index_id +
                            '_%s' % report_age_cat] = index[
                                age_slice[age_category]]

        return result
